#include "scripts/map/map_jam_utils.cpp"
#include "scripts/map/map_jam_defs.cpp"
#include "scripts/map/map_jam_warp_portals.cpp"
#include "scripts/map/map_jam_spirit_world.cpp"
#include "scripts/map/map_jam_credits.cpp"

namespace JAM
{
	const int SCRIPTID_MAP_TICK = 60000;
	const int SCRIPTID_MAP_CREDITSFLY_START = 60001;
	const int SCRIPTID_MAP_CREDITSFLYSKIP_TICK = 60002;
	const int SCRIPTID_MAP_CREDITSENDSKIP_TICK = 60003;
	
	kStr GV_JAM = "JAM";
	kStr GV_JAM_CREDITSMAPINDEX = "JAM_CREDITSMAPINDEX";
	kStr GV_JAM_MAP = "JAM_MAP_";
	kStr GV_MAP_FLAGS = "_FLAGS";
	
	enum GVJamFlags
	{
		GV_JAM_FLAG_CREDITS = (1 << 0), // playing credits sequence
		GV_JAM_FLAG_COMPLETE = (1 << 1), // player has completed the map jam by reaching the end
		GV_JAM_FLAG_COMPLETESOUND = (1 << 2) // played title sounds on game complete
	}
	
	enum GVMapFlags
	{
		GV_MAP_FLAG_PLAYED = (1 << 0) // has started the map at least once before
	}

	void SetupNewGame()
	{
		//Setup one time only on game start
		if (!GameVariables.HasKey(GV_JAM))
		{
			GameVariables.Add(GV_JAM, "0");
			GameVariables.Add(GV_JAM_CREDITSMAPINDEX, "-1");
			for (uint i = 0; i < MAP_DEFS.length(); ++i)
				GameVariables.Add(GV_JAM_MAP + MAP_DEFS[i].id + GV_MAP_FLAGS, "0");
			
			GameVariables.SetValue("g_startmap", START_MAP); //for standalone mod
		}
		else //NewGame
		{
			//Reset GameVars
			GameVarSetFlags(GV_JAM, GV_JAM_FLAG_CREDITS, false);
			GameVariables.SetValue(GV_JAM_CREDITSMAPINDEX, "-1");
			for (uint i = 0; i < MAP_DEFS.length(); ++i)
				GameVariables.SetValue(GV_JAM_MAP + MAP_DEFS[i].id + GV_MAP_FLAGS, "0");
		}
	}

	void SetupMap()
	{
		//teleport to start map if hasn't played it yet
		if (Game.GetCurrentMapID() != START_MAP_ID)
		{
			kStr gvStartMapKey = GV_JAM_MAP + START_MAP_ID + GV_MAP_FLAGS;
			if (!GameVarHasFlags(gvStartMapKey, GV_MAP_FLAG_PLAYED))
			{
				Player.Lock();
				delay(0.0f); //Required or game may trigger an assert when warping to the starting map when this code was already executed in previous map
				PlayLoop.StartWarp(Player.Actor().CastToActor(), START_MAP_WARPTID, START_MAP_ID);
				Game.HaltMapScript(0);
				delay(0.0f); //Need to wait to make this script no longer continue to execute
				return;
			}
		}
		
		SetupWarpPortals();
		SetupSpiritWorld();
		
		//if playing credits then start cinema and do fly through
		if (IsCreditsPlaying())
		{
			Game.CallDelayedMapScript(SCRIPTID_MAP_CREDITSFLY_START, Player.Actor().CastToActor(), 0.0f);
			return;
		}
		
		//if first time in this map then show map name and authors name
		kStr gvCurrentMapKey = GV_JAM_MAP + Game.GetCurrentMapID() + GV_MAP_FLAGS;
		if (!GameVarHasFlags(gvCurrentMapKey, GV_MAP_FLAG_PLAYED))
		{
			GameVarSetFlags(gvCurrentMapKey, GV_MAP_FLAG_PLAYED);
			MapDef@ mapDef = GetMapDef(Game.GetCurrentMapID());
			if (mapDef !is null && mapDef.showCreditsAtStart)
				canShowStartCredits = true;
		}
		
		Game.CallDelayedMapScript(SCRIPTID_MAP_TICK, Player.Actor().CastToActor(), 0.0f);
	}
	
	void TickMap()
	{
		TickMapCreditsStart();
		TickSpiritWorld();
		TickWarpPortals();
	}
	
	void BeatTheGame()
	{
		GameVarSetFlags(GV_JAM, GV_JAM_FLAG_COMPLETE | GV_JAM_FLAG_CREDITS, true);
		
		//go to first credits map
		GameVariables.SetValue(GV_JAM_CREDITSMAPINDEX, "-1");
		WarpToNextCreditsMap();
	}
}

$script 60000 //SCRIPTID_MAP_TICK
{
	JAM::TickMap();
	$restart;
}

$script 60001 //SCRIPTID_MAP_CREDITSFLY_START
{
	JAM::StartMapCreditsFlyThrough(60001);
}

$script 60002 //SCRIPTID_MAP_CREDITSFLYSKIP_TICK
{
	//Set player to camera position so enemies update around the camera
	Player.Actor().CastToActor().Origin() = Camera.origin;

	if (Camera.UserInterrupted())
	{
		JAM::skipCredits = true;
		JAM::HaltCreditScriptCaller();
		JAM::WarpToNextCreditsMap();
		return;
	}
	
	$restart;
}

$script 60003 //SCRIPTID_MAP_CREDITSENDSKIP_TICK
{
	if (Camera.UserInterrupted())
	{
		JAM::skipCredits = true;
		JAM::HaltCreditScriptCaller();
		JAM::EndCredits();
		return;
	}

	$restart;
}
