namespace JAM
{
	const float WARPPORTAL_WAIT_TIME = 30.0f; //Time until warp portal will be shown
	const float WARPPORTAL_HIDE_TIME = 15.0f; //Time until warp portal will be hidden
	const float WARPPORTAL_APPEAR_TIME = 1.0f;
	const float WARPPORTAL_DISAPPEAR_TIME = 1.0f;

	enum EnumWarpPortalStates
	{
		WARPPORTALSTATE_HIDDEN,
		WARPPORTALSTATE_APPEARING,
		WARPPORTALSTATE_IDLE,
		WARPPORTALSTATE_DISAPPEARING,
	}
	
	array<kActor@> warpPortals;
	int warpPortalsState = WARPPORTALSTATE_HIDDEN;
	float warpPortalsTime;
	float warpPortalsScaler;
	
	void SetupWarpPortals()
	{
		SetWarpPortalsState(WARPPORTALSTATE_HIDDEN);
		MapDef@ mapDef = GetMapDef(Game.GetCurrentMapID());
		if (mapDef !is null)
		{
			for (uint i = 0; i < mapDef.warpPortalTIDs.length(); ++i)
				AddWarpPortal(mapDef.warpPortalTIDs[i]);
		}
	}

	kActor@ AddWarpPortal(const int tid)
	{
		kActor@ warpPortal = World.GetActorByTID(tid);
		if (warpPortal !is null)
		{
			warpPortal.Flags() |= AF_HIDDEN;
			warpPortals.insertLast(warpPortal);
		}
		
		return warpPortal;
	}
	
	bool EnterWarpPortal(const int warpTID, const int mapID)
	{
		if (warpPortalsState == WARPPORTALSTATE_IDLE || warpPortalsState == WARPPORTALSTATE_DISAPPEARING)
		{
			delay(0.0f);
			PlayLoop.StartWarp(Player.Actor().CastToActor(), warpTID, mapID);
			return true;
		}
		
		return false;
	}
		
	void SetWarpPortalsState(const int state)
	{
		warpPortalsTime = 0.0f;
		warpPortalsState = state;
		switch (warpPortalsState)
		{
			case WARPPORTALSTATE_HIDDEN:
			{
				for (uint i = 0; i < warpPortals.length(); ++i)
				{
					warpPortals[i].Flags() |= AF_HIDDEN;
					warpPortals[i].StopSound();
				}
				
				break;
			}
			case WARPPORTALSTATE_APPEARING:
			{
				for (uint i = 0; i < warpPortals.length(); ++i)
				{
					warpPortals[i].Flags() &= ~AF_HIDDEN;
				}

				break;
			}
			case WARPPORTALSTATE_IDLE:
			{
				break;
			}
			case WARPPORTALSTATE_DISAPPEARING:
			{
				break;
			}
		}
	}

	void TickWarpPortals()
	{
		warpPortalsTime += GAME_DELTA_TIME;
		switch (warpPortalsState)
		{
			case WARPPORTALSTATE_HIDDEN:
			{
				if (warpPortalsTime >= WARPPORTAL_WAIT_TIME)
				{
					SetWarpPortalsState(WARPPORTALSTATE_APPEARING);
				}
				
				break;
			}
			case WARPPORTALSTATE_APPEARING:
			{
				warpPortalsScaler = Math::Lerp(0.0f, 1.0f, Math::Min(warpPortalsTime / WARPPORTAL_APPEAR_TIME, 1.0f));
				if (warpPortalsTime >= WARPPORTAL_APPEAR_TIME)
				{
					SetWarpPortalsState(WARPPORTALSTATE_IDLE);
				}

				break;
			}
			case WARPPORTALSTATE_IDLE:
			{
				if (warpPortalsTime >= WARPPORTAL_HIDE_TIME)
				{
					SetWarpPortalsState(WARPPORTALSTATE_DISAPPEARING);
				}
				
				break;
			}
			case WARPPORTALSTATE_DISAPPEARING:
			{
				warpPortalsScaler = Math::Lerp(1.0f, 0.0f, Math::Min(warpPortalsTime / WARPPORTAL_DISAPPEAR_TIME, 1.0f));
				if (warpPortalsTime >= WARPPORTAL_DISAPPEAR_TIME)
				{
					SetWarpPortalsState(WARPPORTALSTATE_HIDDEN);
				}
				
				break;
			}
		}
		
		for (uint i = 0; i < warpPortals.length(); ++i)
		{
			kActor@ warpPortal = warpPortals[i];
			if ((warpPortal.Flags() & AF_HIDDEN) == 0)
			{
				warpPortal.PlaySound("sounds/shaders/generic_187.ksnd");
			}
			
			warpPortal.Scale() = kVec3(0.35f, 0.35f, 0.35f) * warpPortalsScaler;
			warpPortal.RenderModel().Offset().z = 200.0f * (1.0f - warpPortalsScaler) * 0.38f;
		}		
	}
}
